﻿using System;
using CustomerManagement.Core.Model.ApplicationEvents;
using CustomerManagement.Core.Services;
using Shared;

namespace CustomerManagement.Core.Model
{
  public class Customer
  {
    public static Customer Create(string name, string source) {
      return new Customer(name, source);
    }
    private Customer(string name, string source){
      Id = Guid.NewGuid();
      Name = name;
      InitialDate = DateTime.UtcNow;
      ModifiedDate = DateTime.UtcNow;
      Source = source;
      PublishEvent (true);
    }

    public Guid Id { get; private set; }
    public string Name { get; private set; }
    public DateTime InitialDate { get; private set; }
    public DateTime ModifiedDate { get; private set; }
    public String Source { get; private set; }
    

    public void FixName(string newName){
      Name = newName;
      ModifiedDate = DateTime.UtcNow;
      PublishEvent(false);
    }

    private void PublishEvent(bool isNew){
      var dto = CustomerDto.Create(Id, Name);
      DomainEvents.Raise(new CustomerUpdatedEvent(dto, isNew));
    }
  }
}